@@ -1,3 +1,4 @@
+import django
 from django.conf import settings
 from django.db import models
 from django.utils.safestring import mark_safe
@@ -141,13 +142,16 @@ class MarkupField(models.TextField):
         setattr(model_instance, _rendered_field_name(self.attname), rendered)
         return value.raw
 
-    def get_db_prep_value(self, value):
-        # for Django 1.2+ rename this to get_prep_value
+    def get_prep_value(self, value):
         if isinstance(value, Markup):
             return value.raw
         else:
             return value
 
+    # copy get_prep_value to get_db_prep_value if pre-1.2
+    if django.VERSION < (1,2):
+        get_db_prep_value = get_prep_value
+
     def value_to_string(self, obj):
         value = self._get_val_from_obj(obj)
         return value.raw
